@@ -17,7 +17,9 @@ class OpenApiArtProtobuf(OpenApiArtPlugin):
         self._write_header(self._openapi["info"])
         for name, schema_object in self._openapi["components"]["schemas"].items():
             self._write_msg(name, schema_object)
-        for url, path_object in self._openapi["paths"].items():
+        for name, response_object in self._openapi["components"]["responses"].items():
+            self._write_msg(name, response_object)
+        for _, path_object in self._openapi["paths"].items():
             self._write_request_msg(path_object)
             self._write_response_msg(path_object)
         self._write_service()
@@ -37,7 +39,7 @@ class OpenApiArtProtobuf(OpenApiArtPlugin):
         return None
 
     def _write_request_msg(self, path_object):
-        for method, path_item_object in path_object.items():
+        for _, path_item_object in path_object.items():
             operation = self._get_operation(path_item_object)
             if operation is None:
                 continue
@@ -48,7 +50,7 @@ class OpenApiArtProtobuf(OpenApiArtPlugin):
                 for ref in self._get_parser('$..requestBody.."$ref"').find(path_item_object):
                     message = self._get_message_name(ref.value)
                     field_type = message.replace(".", "")
-                    field_name = field_type.lower()
+                    field_name = self._lowercase(field_type)
                     self._write("{} {} = 1;".format(field_type, field_name), indent=1)
                 self._write("}")
 
@@ -57,7 +59,7 @@ class OpenApiArtProtobuf(OpenApiArtPlugin):
         application/octet-stream -> response (stream <operationId>Response)
         application/json -> response (<operationId>Response)
         """
-        for method, path_item_object in path_object.items():
+        for _, path_item_object in path_object.items():
             operation = self._get_operation(path_item_object)
             if operation is None:
                 continue
@@ -70,22 +72,26 @@ class OpenApiArtProtobuf(OpenApiArtPlugin):
                     detail_messages.append(detail_message)
                     self._write("message {} {{".format(detail_message), indent=1)
                     schema = self._get_parser("$..schema").find(response)
-                    if len(schema) != 0:
+                    if len(schema) == 0:
+                        if "$ref" in response:
+                            field_type = self._get_message_name(response["$ref"]).replace(".", "")
+                            self._write("{} {} = 1;".format(field_type, self._lowercase(field_type)), indent=2)
+                    else:
                         schema = schema[0].value
-                    if "$ref" in schema:
-                        field_type = self._get_message_name(schema["$ref"]).replace(".", "")
-                        self._write("{} {} = 1;".format(field_type, field_type.lower()), indent=2)
-                    elif "type" in schema:
-                        field_type = self._get_message_name(schema["type"]).replace(".", "")
-                        if "format" in schema and schema["format"] == "binary":
-                            field_type = "bytes"
-                        self._write("{} {} = 1;".format(field_type, field_type.lower()), indent=2)
+                        if "$ref" in schema:
+                            field_type = self._get_message_name(schema["$ref"]).replace(".", "")
+                            self._write("{} {} = 1;".format(field_type, self._lowercase(field_type)), indent=2)
+                        elif "type" in schema:
+                            field_type = self._get_message_name(schema["type"]).replace(".", "")
+                            if "format" in schema and schema["format"] == "binary":
+                                field_type = "bytes"
+                            self._write("{} {} = 1;".format(field_type, self._lowercase(field_type)), indent=2)
                     self._write("}", indent=1)
             self._write("oneof statuscode {", indent=1)
             id = 1
             for detail_message in detail_messages:
                 field_type = detail_message.replace(".", "")
-                field_name = field_type.lower().replace("-", "").replace("_", "")
+                field_name = self._lowercase(field_type)
                 self._write("{} {} = {};".format(field_type, field_name, id), indent=2)
                 id += 1
             self._write("}", indent=1)
@@ -99,6 +105,7 @@ class OpenApiArtProtobuf(OpenApiArtPlugin):
         return self._custom_id
 
     def _write_header(self, info_object):
+        self._write("// {}".format(self._info))
         for line in self._license.split("\n"):
             self._write("// {}".format(line))
         self._write()
@@ -160,9 +167,9 @@ class OpenApiArtProtobuf(OpenApiArtPlugin):
                     if openapi_object["format"] == "binary":
                         return "bytes"
                 elif "enum" in openapi_object:
-                    enum_msg = self._camelcase("{}.Enum".format(property_name))
+                    enum_msg = self._camelcase("{}".format(property_name))
                     self._write_enum_msg(enum_msg, openapi_object["enum"])
-                    return enum_msg
+                    return enum_msg + ".Enum"
                 return "string"
             if type == "integer":
                 return "int32"
@@ -174,7 +181,33 @@ class OpenApiArtProtobuf(OpenApiArtPlugin):
             return openapi_object["$ref"].split("/")[-1].replace(".", "")
 
     def _camelcase(self, value):
-        return "{}{}".format(value[0].upper(), value[1:])
+        camel_case = ""
+        for piece in value.split("_"):
+            camel_case += "{}{}".format(piece[0].upper(), piece[1:])
+        return camel_case
+
+    def _camelcase_to_snakecase(self, value, lower=False):
+        word = ""
+        insert_underscore = False
+
+        for c in value:
+            if c.isupper() or c.isdigit():
+                if insert_underscore:
+                    word += "_" + (c.lower() if lower else c)
+                    insert_underscore = False
+                else:
+                    word += c.lower() if lower else c
+            else:
+                word += c if lower else c.upper()
+                insert_underscore = True
+
+        return word
+
+    def _uppercase(self, value):
+        return self._camelcase_to_snakecase(value, lower=False)
+
+    def _lowercase(self, value):
+        return self._camelcase_to_snakecase(value, lower=True)
 
     def _get_description(self, openapi_object):
         if "description" in openapi_object:
@@ -183,12 +216,15 @@ class OpenApiArtProtobuf(OpenApiArtPlugin):
             return "Description missing in models"
 
     def _write_enum_msg(self, enum_msg_name, enums):
-        enum_msg = enum_msg_name.split(".")
-        self._write("message {} {{".format(enum_msg[0]), indent=1)
+        """Follow google developers style guide for enums
+        - reference: https://developers.google.com/protocol-buffers/docs/style#enums
+        """
+        self._write("message {} {{".format(enum_msg_name.replace(".", "")), indent=1)
         self._write("enum Enum {", indent=2)
+        enums.insert(0, "unspecified")
         id = 0
         for enum in enums:
-            self._write("{} = {};".format(enum.upper(), id), indent=3)
+            self._write("{} = {};".format(enum.lower(), id), indent=3)
             id += 1
         self._write("}", indent=2)
         self._write("}", indent=1)
@@ -199,9 +235,22 @@ class OpenApiArtProtobuf(OpenApiArtPlugin):
         self._write()
         self._write("message {} {{".format(msg_name), indent=0)
         self._write('option (msg_meta).description = "{}";'.format(self._get_description(schema_object)), indent=1)
-        self._write_msg_fields(name, schema_object)
+        if "content" in schema_object:
+            # when accessing components/responses
+            self._write_response_fields(msg_name, schema_object)
+        else:
+            # when accessing components/schemas
+            self._write_msg_fields(name, schema_object)
         self._write("}")
 
+    def _write_response_fields(self, msg_name, schema_object):
+        try:
+            ref = schema_object["content"]["application/json"]["schema"]["$ref"]
+            field_type = self._get_message_name(ref).replace(".", "")
+            self._write("{} {} = 1;".format(field_type, self._lowercase(field_type)), indent=1)
+        except AttributeError as err:
+            print("Failed writing response {}: {}".format(msg_name, err))
+
     def _write_msg_fields(self, name, schema_object):
         if "properties" not in schema_object:
             return
@@ -213,9 +262,9 @@ class OpenApiArtProtobuf(OpenApiArtPlugin):
             default = None
             if "default" in property_object:
                 default = property_object["default"]
-            if property_type.endswith(".Enum") and default is not None:
-                type = "enum"
-                default = "{}.{}".format(property_type.split(" ")[-1], default.upper())
+            if property_type.endswith(".Enum"):
+                if default is not None:
+                    default = "{}.{}".format(property_type.split(" ")[-1], default.lower())
             if "required" in schema_object and property_name in schema_object["required"] or property_type.startswith("repeated"):
                 optional = ""
             else:
