@@ -31,16 +31,27 @@ def openapiart():
     return openapiart
 
 
-@pytest.fixture
+@pytest.fixture(scope="session")
 def api(openapiart):
     """Return an instance of the top level Api class from the generated package"""
+    from .server import OpenApiServer
     sys.path.append(openapiart.output_dir)
     module = importlib.import_module(openapiart.python_module_name)
-    package = getattr(module, openapiart.python_module_name)
-    return package.api(location=None, verify=False, logger=None, loglevel=logging.DEBUG)
+    # package = getattr(module, openapiart.python_module_name)
+    pytest.server = OpenApiServer(module).start()
+    return module.api(location='http://127.0.0.1:80', verify=False, logger=None, loglevel=logging.DEBUG)
 
 
 @pytest.fixture
 def config(api):
     """Return a new instance of an empty config"""
     return api.prefix_config()
+
+
+@pytest.fixture
+def default_config(api):
+    config = api.prefix_config()
+    config.a = "asdf"
+    config.b = 1.1
+    config.c = 1
+    return config
